#include <propeller.h>
#include "ws2812.h"

static int ticks_per_ms;

int ws2812_init(int pin, uint32_t *colors, int count)
{
    extern uint32_t binary_ws2812_driver_dat_start[];
    ws2812_hdr *hdr = (ws2812_hdr *)binary_ws2812_driver_dat_start;
    ws2812_params params;
    uint32_t ustix;
    int cog;

    if (CLKFREQ != 80000000)            // requires 80MHz clock
        return -1;

    ticks_per_ms = CLKFREQ / 1000;      // ticks per millisecond

    ustix = CLKFREQ / 1000000;          // ticks in 1us

    hdr->resettix = ustix * 50;         // 50.00us min reset timing
    hdr->bit0hi   = ustix * 35 / 100;   //  0.35us
    hdr->bit0lo   = ustix * 80 / 100;   //  0.80us
    hdr->bit1hi   = ustix * 70 / 100;   //  0.70us
    hdr->bit1lo   = ustix * 60 / 100;   //  0.60us
    
    params.pin = pin;
    params.colors = colors;
    params.count = count;
    
    if ((cog = cognew(hdr, &params)) < 0)
        return -1;
        
    while (params.count)
        ;
     
    return cog;
}

void ws2812_fill(uint32_t *colors, int count, uint32_t color)
{
    int i;
    for (i = 0; i < count; ++i)
        colors[i] = color;
}

void ws2812_fill_wheel(uint32_t *colors, int count, int delay)
{
    int i;
    for (i = 0; i < count; ++i) {
        colors[i] = ws2812_wheel(i * (255 / count));
        ws2812_pause(delay);
    }
}

void ws2812_fill_wheel_dim(uint32_t *colors, int count, int delay, int brightness)
{
    int i;
    for (i = 0; i < count; ++i) {
        colors[i] = ws2812_wheel_dim(i * (255 / count), brightness);
        ws2812_pause(delay);
    }
}

void ws2812_fill_wheel_continuous(uint32_t *colors, int count, int samples, int delay)
{
    int i = 0, di = 1;
    int j = 0;
    for (;;) {
        colors[i] = ws2812_wheel(j * (255 / samples));
        ws2812_pause(delay);
        i += di;
        if (i >= count) {
            di = -1;
            --i;
        }
        else if (i < 0) {
            di = 1;
            ++i;
        } 
        if (++j >= samples)
            j = 0;    
    }
}

void ws2812_fill_wheel_continuous_dim(uint32_t *colors, int count, int samples, int brightness, int delay)
{
    int i = 0, di = 1;
    int j = 0;
    for (;;) {
        colors[i] = ws2812_wheel_dim(j * (255 / samples), brightness);
        ws2812_pause(delay);
        i += di;
        if (i >= count) {
            di = -1;
            --i;
        }
        else if (i < 0) {
            di = 1;
            ++i;
        } 
        if (++j >= samples)
            j = 0;    
    }
}

uint32_t ws2812_wheel(int pos)
{
    uint32_t color;

// Creates color from 0 to 255 position input
// -- colors transition r-g-b back to r

    // red range
    if (pos < 85)
        color = COLOR(255-pos*3, pos*3, 0);
    
    // green range
    else if (pos < 170) {
        pos -= 85;
        color = COLOR(0, 255-pos*3, pos*3);
    }
    
    // blue range
    else {
        pos -= 170;
        color = COLOR(pos*3, 0, 255-pos*3);
    }
    
    return color;
}

uint32_t ws2812_wheel_dim(int pos, int brightness)
{
    uint32_t color;

// Creates color from 0 to 255 position input
// -- colors transition r-g-b back to r

    // red range
    if (pos < 85)
        color = COLORX(255-pos*3, pos*3, 0, brightness);
    
    // green range
    else if (pos < 170) {
        pos -= 85;
        color = COLORX(0, 255-pos*3, pos*3, brightness);
    }
    
    // blue range
    else {
        pos -= 170;
        color = COLORX(pos*3, 0, 255-pos*3, brightness);
    }
    
    return color;
}

void ws2812_pause(int ms)
{
    waitcnt(CNT + ms * ticks_per_ms);
}